#include <rclcpp/rclcpp.hpp>
#include <move_group_interface/srv/move_pose.hpp>

class ScanNPlan : public rclcpp::Node {
public:
    ScanNPlan() : Node("scan_n_plan") {
        move_client_ = this->create_client<move_group_interface::srv::MovePose>("update_pose");
    }

    void start() {
        RCLCPP_INFO(get_logger(), "Attempting to move to pose");

        // Wait for service to be available
        if (!move_client_->wait_for_service(std::chrono::seconds(5))) {
            RCLCPP_ERROR(get_logger(), "Unable to find move_pose service. Start move_pose first.");
            return;
        }

        // Create a request for the MovePose service call
        auto request = std::make_shared<move_group_interface::srv::MovePose::Request>();

        auto future = move_client_->async_send_request(request);

        if (rclcpp::spin_until_future_complete(this->get_node_base_interface(), future) != rclcpp::executor::FutureReturnCode::SUCCESS) {
            RCLCPP_ERROR(this->get_logger(), "Failed to receive MovePose service response");
            return;
        }

        auto response = future.get();
        if (!response->success) {
            RCLCPP_ERROR(this->get_logger(), "MovePose service failed");
            return;
        }

        RCLCPP_INFO(this->get_logger(), "Moved to pose: x: %f, y: %f, z: %f - x: %f, y: %f, z: %f, w: %f",
            response->pose.position.x,
            response->pose.position.y,
            response->pose.position.z,

            response->pose.orientation.x,
            response->pose.orientation.y,
            response->pose.orientation.z,
            response->pose.orientation.w
        );
    }
private:
    rclcpp::Client<move_group_interface::srv::MovePose>::SharedPtr move_client_;

};

int main(int argc, char **argv) {
    rclcpp::init(argc, argv);

    auto app = std::make_shared<ScanNPlan>();
    app->start();

    rclcpp::shutdown();
    return 0;
}
