#include <rclcpp/rclcpp.hpp>
#include "geometry_msgs/msg/pose_stamped.hpp"

#include <move_group_interface/srv/move_pose.hpp>

class Localizer : public rclcpp::Node {
    public:
        Localizer() : Node("move_group_interface_node"), last_msg_{nullptr} {
            pose_sub_ = this->create_subscription<geometry_msgs::msg::PoseStamped>(
                "move_pose",
                rclcpp::QoS(1),
                std::bind(&Localizer::moveCallback, this, std::placeholders::_1)
            );

            server_ = this->create_service<move_group_interface::srv::MovePose>(
                "update_pose",
                std::bind(&Localizer::movePose, this, std::placeholders::_1, std::placeholders::_2)
            );
        }

        void moveCallback(geometry_msgs::msg::PoseStamped::SharedPtr msg) {
            last_msg_ = msg;
        }

        void movePose(move_group_interface::srv::MovePose::Request::SharedPtr req,
                      move_group_interface::srv::MovePose::Response::SharedPtr res) {
            // Read last message
            geometry_msgs::msg::PoseStamped::SharedPtr p = last_msg_;

            if (!p) {
                RCLCPP_ERROR(this->get_logger(), "no data");
                res->success = false;
                return;
            }

            res->header = p->header;
            res->pose = p->pose;
            res->success = true;
        }

        rclcpp::Subscription<geometry_msgs::msg::PoseStamped>::SharedPtr pose_sub_;
        geometry_msgs::msg::PoseStamped::SharedPtr last_msg_;
        rclcpp::Service<move_group_interface::srv::MovePose>::SharedPtr server_;
};

int main(int argc, char* argv[]) {
    rclcpp::init(argc, argv);

    auto node = std::make_shared<Localizer>();

    RCLCPP_INFO(node->get_logger(), "MoveGroup node started.");

    rclcpp::spin(node);
}
